"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.prepareInterceptor = prepareInterceptor;
const Interceptor = Symbol('Interceptor for programmatical calls');

function prepareInterceptor(element, propName, interceptorImpl) {
  const prototypeDescriptor = Object.getOwnPropertyDescriptor(element.constructor.prototype, propName);
  const target = prototypeDescriptor != null && prototypeDescriptor.set ? 'set' : 'value';

  if (typeof (prototypeDescriptor == null ? void 0 : prototypeDescriptor[target]) !== 'function' || prototypeDescriptor[target][Interceptor]) {
    return;
  }

  const realFunc = prototypeDescriptor[target];

  function intercept(...args) {
    const realArgs = interceptorImpl.call(this, ...args);

    if (target === 'set') {
      realFunc.call(this, realArgs);
    } else {
      realFunc.call(this, ...realArgs);
    }
  }

  ;
  intercept[Interceptor] = Interceptor;
  Object.defineProperty(element.constructor.prototype, propName, { ...prototypeDescriptor,
    [target]: intercept
  });
}