"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getInitialValue = getInitialValue;
exports.getUIValue = getUIValue;
exports.prepareValueInterceptor = prepareValueInterceptor;
exports.setInitialValue = setInitialValue;
exports.setUIValue = setUIValue;

var _applyNative = require("./applyNative");

var _interceptor = require("./interceptor");

var _selection = require("./selection");

const UIValue = Symbol('Displayed value in UI');
const InitialValue = Symbol('Initial value to compare on blur');

function valueInterceptor(v) {
  const isUI = typeof v === 'object' && v[UIValue];
  this[UIValue] = isUI ? String(v) : undefined;

  if (!isUI) {
    this[InitialValue] = String(v); // Programmatically setting the value property
    // moves the cursor to the end of the input.

    (0, _selection.clearUISelection)(this);
  }

  return String(v);
}

function prepareValueInterceptor(element) {
  (0, _interceptor.prepareInterceptor)(element, 'value', valueInterceptor);
}

function setUIValue(element, value) {
  (0, _applyNative.applyNative)(element, 'value', {
    [UIValue]: UIValue,
    toString: () => value
  });
}

function getUIValue(element) {
  return element[UIValue] === undefined ? element.value : element[UIValue];
}

function setInitialValue(element) {
  element[InitialValue] = element.value;
}

function getInitialValue(element) {
  return element[InitialValue];
}