"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.pointerMove = pointerMove;

var _utils = require("../utils");

async function pointerMove({
  pointerName = 'mouse',
  target,
  coords
}, {
  pointerState,
  keyboardState
}) {
  if (!(pointerName in pointerState.position)) {
    throw new Error(`Trying to move pointer "${pointerName}" which does not exist.`);
  }

  const {
    pointerId,
    pointerType,
    target: prevTarget,
    coords: prevCoords
  } = pointerState.position[pointerName];

  if (prevTarget && prevTarget !== target) {
    // Here we could probably calculate a few coords to a fake boundary(?)
    fireMove(prevTarget, prevCoords);

    if (!(0, _utils.isDescendantOrSelf)(target, prevTarget)) {
      fireLeave(prevTarget, prevCoords);
    }
  }

  pointerState.position[pointerName] = {
    pointerId,
    pointerType,
    target,
    coords
  };

  if (prevTarget !== target) {
    if (!prevTarget || !(0, _utils.isDescendantOrSelf)(prevTarget, target)) {
      fireEnter(target, coords);
    }
  } // TODO: drag if the target didn't change?
  // Here we could probably calculate a few coords leading up to the final position


  fireMove(target, coords);

  function fireMove(eventTarget, eventCoords) {
    fire(eventTarget, 'pointermove', eventCoords);

    if (pointerType === 'mouse' && !(0, _utils.isDisabled)(eventTarget)) {
      fire(eventTarget, 'mousemove', eventCoords);
    }
  }

  function fireLeave(eventTarget, eventCoords) {
    fire(eventTarget, 'pointerout', eventCoords);
    fire(eventTarget, 'pointerleave', eventCoords);

    if (pointerType === 'mouse' && !(0, _utils.isDisabled)(eventTarget)) {
      fire(eventTarget, 'mouseout', eventCoords);
      fire(eventTarget, 'mouseleave', eventCoords);
    }
  }

  function fireEnter(eventTarget, eventCoords) {
    fire(eventTarget, 'pointerover', eventCoords);
    fire(eventTarget, 'pointerenter', eventCoords);

    if (pointerType === 'mouse' && !(0, _utils.isDisabled)(eventTarget)) {
      fire(eventTarget, 'mouseover', eventCoords);
      fire(eventTarget, 'mouseenter', eventCoords);
    }
  }

  function fire(eventTarget, type, eventCoords) {
    return (0, _utils.firePointerEvent)(eventTarget, type, {
      pointerState,
      keyboardState,
      coords: eventCoords,
      pointerId,
      pointerType
    });
  }
}