"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.pointerPress = pointerPress;

var _dom = require("@testing-library/dom");

var _utils = require("../utils");

/* eslint-disable complexity */
async function pointerPress({
  keyDef,
  releasePrevious,
  releaseSelf,
  target,
  coords
}, state) {
  const previous = state.pointerState.pressed.find(p => p.keyDef === keyDef);
  const pointerName = keyDef.pointerType === 'touch' ? keyDef.name : keyDef.pointerType;
  const targetIsDisabled = (0, _utils.isDisabled)(target);

  if (previous) {
    up(state, pointerName, keyDef, targetIsDisabled, target, coords, previous);
  }

  if (!releasePrevious) {
    const press = down(state, pointerName, keyDef, targetIsDisabled, target, coords);

    if (releaseSelf) {
      up(state, pointerName, keyDef, targetIsDisabled, target, coords, press);
    }
  }
}

function getNextPointerId(state) {
  state.pointerId = state.pointerId + 1;
  return state.pointerId;
}

function down({
  pointerState,
  keyboardState
}, pointerName, keyDef, targetIsDisabled, target, coords) {
  var _pointerState$activeC, _pointerState$activeC2, _pointerState$activeC3;

  const {
    name,
    pointerType,
    button
  } = keyDef;
  const pointerId = pointerType === 'mouse' ? 1 : getNextPointerId(pointerState);
  pointerState.position[pointerName] = {
    pointerId,
    pointerType,
    target,
    coords
  };
  let isMultiTouch = false;
  let isPrimary = true;

  if (pointerType !== 'mouse') {
    for (const obj of pointerState.pressed) {
      // TODO: test multi device input across browsers
      // istanbul ignore else
      if (obj.keyDef.pointerType === pointerType) {
        obj.isMultiTouch = true;
        isMultiTouch = true;
        isPrimary = false;
      }
    }
  }

  if (((_pointerState$activeC = pointerState.activeClickCount) == null ? void 0 : _pointerState$activeC[0]) !== name) {
    delete pointerState.activeClickCount;
  }

  const clickCount = Number((_pointerState$activeC2 = (_pointerState$activeC3 = pointerState.activeClickCount) == null ? void 0 : _pointerState$activeC3[1]) != null ? _pointerState$activeC2 : 0) + 1;
  pointerState.activeClickCount = [name, clickCount];
  const pressObj = {
    keyDef,
    downTarget: target,
    pointerId,
    unpreventedDefault: true,
    isMultiTouch,
    isPrimary,
    clickCount
  };
  pointerState.pressed.push(pressObj);

  if (!targetIsDisabled) {
    if (pointerType !== 'mouse') {
      fire('pointerover');
      fire('pointerenter');
    }

    if (pointerType !== 'mouse' || !pointerState.pressed.some(p => p.keyDef !== keyDef && p.keyDef.pointerType === pointerType)) {
      fire('pointerdown');
    }

    if (pointerType === 'mouse') {
      pressObj.unpreventedDefault = fire('mousedown');
    } // TODO: touch...

  }

  if (pointerType === 'mouse' && pressObj.unpreventedDefault) {
    var _findClosest;

    (0, _utils.focus)((_findClosest = (0, _utils.findClosest)(target, _utils.isFocusable)) != null ? _findClosest : target.ownerDocument.body);
  }

  return pressObj;

  function fire(type) {
    return (0, _utils.firePointerEvent)(target, type, {
      pointerState,
      keyboardState,
      button,
      clickCount,
      coords,
      isPrimary,
      pointerId,
      pointerType
    });
  }
}

function up({
  pointerState,
  keyboardState
}, pointerName, {
  pointerType,
  button
}, targetIsDisabled, target, coords, pressed) {
  pointerState.pressed = pointerState.pressed.filter(p => p !== pressed);
  const {
    isMultiTouch,
    isPrimary,
    pointerId,
    clickCount
  } = pressed;
  let {
    unpreventedDefault
  } = pressed;
  pointerState.position[pointerName] = {
    pointerId,
    pointerType,
    target,
    coords
  }; // TODO: pointerleave for touch device

  if (!targetIsDisabled) {
    if (pointerType !== 'mouse' || !pointerState.pressed.filter(p => p.keyDef.pointerType === pointerType).length) {
      fire('pointerup');
    }

    if (pointerType !== 'mouse') {
      fire('pointerout');
      fire('pointerleave');
    }

    if (pointerType !== 'mouse' && !isMultiTouch) {
      if (clickCount === 1) {
        fire('mouseover');
        fire('mouseenter');
      }

      fire('mousemove');
      unpreventedDefault = fire('mousedown') && unpreventedDefault;
    }
  }

  if (unpreventedDefault && pointerType !== 'mouse' && !isMultiTouch) {
    var _findClosest2;

    // The closest focusable element is focused when a `mousedown` would have been fired.
    // Even if there was no `mousedown` because the element was disabled.
    // A `mousedown` that preventsDefault cancels this though.
    (0, _utils.focus)((_findClosest2 = (0, _utils.findClosest)(target, _utils.isFocusable)) != null ? _findClosest2 : target.ownerDocument.body);
  }

  if (!targetIsDisabled) {
    if (pointerType === 'mouse' || !isMultiTouch) {
      unpreventedDefault = fire('mouseup') && unpreventedDefault;
      const canClick = pointerType !== 'mouse' || button === 'primary';

      if (canClick && target === pressed.downTarget) {
        fire('click');

        if (clickCount === 2) {
          fire('dblclick');
        } // If the click happens inside a `label` with a control, the control (or closes focusable) is focused.


        const label = target.closest('label');

        if (label != null && label.control) {
          var _findClosest3;

          (0, _utils.focus)((_findClosest3 = (0, _utils.findClosest)(label.control, _utils.isFocusable)) != null ? _findClosest3 : target.ownerDocument.body);
        }
      }

      if (pointerType === 'mouse' && button === 'secondary') {
        _dom.fireEvent.contextMenu(target);
      }
    }
  }

  function fire(type) {
    return (0, _utils.firePointerEvent)(target, type, {
      pointerState,
      keyboardState,
      button,
      clickCount,
      coords,
      isPrimary,
      pointerId,
      pointerType
    });
  }
}