"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fireInputEvent = fireInputEvent;

var _dom = require("@testing-library/dom");

var _isElementType = require("../misc/isElementType");

var _document = require("../../document");

var _isContentEditable = require("./isContentEditable");

var _selectionRange = require("./selectionRange");

function fireInputEvent(element, {
  newValue,
  newSelectionStart,
  eventOverrides
}) {
  // apply the changes before firing the input event, so that input handlers can access the altered dom and selection
  if ((0, _isContentEditable.isContentEditable)(element)) {
    (0, _document.applyNative)(element, 'textContent', newValue);
  } else
    /* istanbul ignore else */
    if ((0, _isElementType.isElementType)(element, ['input', 'textarea'])) {
      (0, _document.setUIValue)(element, newValue);
    } else {
      // TODO: properly type guard
      throw new Error('Invalid Element');
    }

  setSelectionRangeAfterInput(element, newSelectionStart);

  _dom.fireEvent.input(element, { ...eventOverrides
  });

  setSelectionRangeAfterInputHandler(element, newSelectionStart);
}

function setSelectionRangeAfterInput(element, newSelectionStart) {
  (0, _selectionRange.setSelectionRange)(element, newSelectionStart, newSelectionStart);
}

function setSelectionRangeAfterInputHandler(element, newSelectionStart) {
  // On controlled inputs the selection changes without a call to
  // either the `value` setter or the `setSelectionRange` method.
  // So if our tracked position for UI still exists and derives from a valid selectionStart,
  // the cursor was moved due to an input being controlled.
  if ((0, _isElementType.isElementType)(element, ['input', 'textarea']) && typeof element.selectionStart === 'number' && element.selectionStart !== newSelectionStart && (0, _document.hasUISelection)(element)) {
    (0, _selectionRange.setSelectionRange)(element, newSelectionStart, newSelectionStart);
  }
}