"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.FakePointerEvent = exports.FakeMouseEvent = void 0;

// See : https://github.com/testing-library/react-testing-library/issues/268
function assignProps(obj, props) {
  for (const [key, value] of Object.entries(props)) {
    Object.defineProperty(obj, key, {
      get: () => value
    });
  }
}

function assignPositionInit(obj, {
  x,
  y,
  clientX,
  clientY,
  offsetX,
  offsetY,
  pageX,
  pageY
}) {
  assignProps(obj, {
    x,
    y,
    clientX,
    clientY,
    offsetX,
    offsetY,
    pageX,
    pageY
  });
}

function assignPointerInit(obj, {
  isPrimary,
  pointerId,
  pointerType
}) {
  assignProps(obj, {
    isPrimary,
    pointerId,
    pointerType
  });
}

const notBubbling = ['mouseenter', 'mouseleave', 'pointerenter', 'pointerleave'];

function getInitDefaults(type, init) {
  return {
    bubbles: !notBubbling.includes(type),
    cancelable: true,
    composed: true,
    ...init
  };
}

class FakeMouseEvent extends MouseEvent {
  constructor(type, init) {
    super(type, getInitDefaults(type, init));
    assignPositionInit(this, init);
  }

} // Should extend PointerEvent, but... https://github.com/jsdom/jsdom/issues/2527


exports.FakeMouseEvent = FakeMouseEvent;

class FakePointerEvent extends MouseEvent {
  constructor(type, init) {
    super(type, getInitDefaults(type, init));
    assignPositionInit(this, init);
    assignPointerInit(this, init);
  }

}

exports.FakePointerEvent = FakePointerEvent;